/*#######################################################################################
AVR Programm to controll the M64282 CMOS image sensor 

Copyright (C) 2005 Ulrich Radig

#######################################################################################*/

#include "main.h"
#include "cam.h"

/*#######################################################################################
#	Init Camera
#######################################################################################*/
void Init_Cam (void)
{
	CAM_DDR |= (1<<CAM_START); //Set CAM_START pin as output
	CAM_DDR |= (1<<CAM_SIN); //Set CAM_SIN pin as output
	CAM_DDR |= (1<<CAM_LOAD); //Set CAM_LOAD pin as output
	CAM_DDR |= (1<<CAM_RESET); //Set CAM_RESET pin as output
	CAM_DDR |= (1<<CAM_CLOCK); //Set CAM_CLOCK pin as output
	CAM_DDR &= ~(1<<CAM_READ); //Set CAM_READ pin as input
	
	//Set Analog to Digital Converter (PORT ADC0 / CAM_Vin)
	ADMUX = 0;
	ADMUX |= (1<<REFS0);
	//ADMUX |= (1<<REFS1);
	ADMUX |= (1<<ADLAR);
	
	ADCSRA = 0;
	ADCSRA |= (1<<ADFR);
	ADCSRA |= (1<<ADSC);
	ADCSRA |= (1<<ADEN);

	//Reset Sequenz (Reset Camera)
	CAM_RESET_LOW();
	CAM_CLOCK_LOW();
	for(char a=0;a<100;a++){nop();};
	CAM_CLOCK_HIGH();
	for(char a=0;a<100;a++){nop();};
	CAM_CLOCK_LOW();
	CAM_RESET_HIGH();
	
	//Set camera registers(0-7)
	unsigned char reg[7];
 	struct  Camera_Register *reg_pointer;
	reg_pointer = (struct Camera_Register *)&reg[0];

	reg_pointer->G = 7;
	reg_pointer->VH = 3;
	reg_pointer->C1 = 25;
	reg_pointer->C0 = 0;
	reg_pointer->P = 1;
	reg_pointer->M = 0;
	reg_pointer->X = 1;
	reg_pointer->I = 0;
	reg_pointer->O = 31;
	reg_pointer->Z = 2;
	reg_pointer->E = 2;
	reg_pointer->V = 7;
	
	
	for (unsigned char a=0;a<8;a++)
		{
		Cam_Data_Load_Seq(a,reg[a]);
		};
	
	//Camera start
	CAM_START_HIGH();
	for(char a=0;a<10;a++){nop();};
	CAM_CLOCK_HIGH();
	for(char a=0;a<10;a++){nop();};
	CAM_CLOCK_LOW();
	CAM_START_LOW();

};

/*#######################################################################################
#	Camera data LOAD sequence
#######################################################################################*/
void Cam_Data_Load_Seq (unsigned char addr,unsigned char data)
{
	//Write 3bit address to camera
	for(char b=0;b<3;b++)
	{
		CAM_CLOCK_LOW();
		if ((addr & 0x04) > LOW)
		{
			CAM_SIN_HIGH();
		}
		else
		{
			CAM_SIN_LOW();
		};
		addr = (addr<<1);
		for(char a=0;a<40;a++){nop();};
		CAM_CLOCK_HIGH();
		for(char a=0;a<40;a++){nop();};
	};
	
	//Write 8bit data to camera
	for(char b=0;b<8;b++)
	{
		CAM_CLOCK_LOW();
		if ((data & BIT8) > LOW)
		{
			CAM_SIN_HIGH();
		}
		else
		{
			CAM_SIN_LOW();
		};
		data = (data<<1);	
		for(char a=0;a<40;a++){nop();};
		CAM_CLOCK_HIGH();
		for(char a=0;a<40;a++){nop();};
	};
	
	CAM_LOAD_HIGH();
	for(char a=0;a<40;a++){nop();};
	CAM_CLOCK_LOW();
	for(char a=0;a<40;a++){nop();};
	CAM_LOAD_LOW();
	CAM_SIN_LOW();
};





